<?php
/*
Plugin Name: TablePress Extension: Display random rows
Plugin URI: https://tablepress.org/extensions/random-row/
Description: Custom Extension for TablePress to display a random set of rows
Version: 1.2
Author: Tobias Bäthge
Author URI: https://tobias.baethge.com/
*/

// [table id=123 show_rows=random random_rows=10 /]

add_filter( 'tablepress_shortcode_table_default_shortcode_atts', 'tablepress_add_shortcode_parameters_random_rows' );
add_filter( 'tablepress_table_render_options', 'tablepress_show_random_row', 10, 2 );

/**
 * Add `random_rows` as a valid Shortcode parameter.
 *
 * @since 1.0
 *
 * @param array $default_atts Current set of Shortcode parameters.
 * @return array Extended set of Shortcode parameters.
 */
function tablepress_add_shortcode_parameters_random_rows( $default_atts ) {
	$default_atts['random_rows'] = 1;
	return $default_atts;
}

/**
 * Show only a certain number of random rows.
 *
 * @since 1.0
 *
 * @param array $render_options Render options for the table.
 * @param array $table          The table.
 * @return array The modified render options.
 */
function tablepress_show_random_row( $render_options, $table ) {
	if ( 'random' === $render_options['show_rows'] ) {
		// Turn off caching and hide all rows by default.
		$render_options['cache_table_output'] = false;
		$render_options['hide_rows'] = 'all';

		// Show the table head row if the option is set.
		$render_options['show_rows'] = array();
		if ( $render_options['table_head'] ) {
			$min = 2;
			$render_options['show_rows'][] = '1';
		} else {
			$min = 1;
		}
		// Show the table foot row if the option is set.
		if ( $render_options['table_foot'] ) {
			$max = count( $table['data'] ) - 1;
			$render_options['show_rows'][] = (string) count( $table['data'] );
		} else {
			$max = count( $table['data'] );
		}

		// Get the desired number or random row numbers.
		$rows = range( $min, $max );
		$rows = array_flip( $rows ); // We want to retrieve random array values, not keys, which array_rand() does.
		$random_rows = (array) array_rand( $rows, (int) $render_options['random_rows'] );

		// Add random rows to the set of shown rows.
		$render_options['show_rows'] = array_merge( $render_options['show_rows'], $random_rows );
		$render_options['show_rows'] = implode( ',', $render_options['show_rows'] );
	}
	return $render_options;
}
